"""Python bindings to libpostal near_dupe_hashes."""

from postal import _dedupe
from postal.utils.enum import Enum, EnumValue


class duplicate_status(Enum):
    NULL_DUPLICATE = EnumValue(_dedupe.NULL_DUPLICATE_STATUS)
    NON_DUPLICATE = EnumValue(_dedupe.NON_DUPLICATE)
    NEEDS_REVIEW = EnumValue(_dedupe.POSSIBLE_DUPLICATE_NEEDS_REVIEW)
    LIKELY_DUPLICATE = EnumValue(_dedupe.LIKELY_DUPLICATE)
    EXACT_DUPLICATE = EnumValue(_dedupe.EXACT_DUPLICATE)


def place_languages(labels, values):
    return _dedupe.place_languages(labels, values)


def is_name_duplicate(value1, value2, languages=None):
    dupe_status = _dedupe.is_name_duplicate(value1, value2, languages=languages)
    return duplicate_status.from_id(dupe_status)


def is_street_duplicate(value1, value2, languages=None):
    dupe_status = _dedupe.is_street_duplicate(value1, value2, languages=languages)
    return duplicate_status.from_id(dupe_status)


def is_house_number_duplicate(value1, value2, languages=None):
    dupe_status = _dedupe.is_house_number_duplicate(value1, value2, languages=languages)
    return duplicate_status.from_id(dupe_status)


def is_po_box_duplicate(value1, value2, languages=None):
    dupe_status = _dedupe.is_po_box_duplicate(value1, value2, languages=languages)
    return duplicate_status.from_id(dupe_status)


def is_unit_duplicate(value1, value2, languages=None):
    dupe_status = _dedupe.is_unit_duplicate(value1, value2, languages=languages)
    return duplicate_status.from_id(dupe_status)


def is_floor_duplicate(value1, value2, languages=None):
    dupe_status = _dedupe.is_floor_duplicate(value1, value2, languages=languages)
    return duplicate_status.from_id(dupe_status)


def is_postal_code_duplicate(value1, value2, languages=None):
    dupe_status = _dedupe.is_postal_code_duplicate(value1, value2, languages=languages)
    return duplicate_status.from_id(dupe_status)


def is_toponym_duplicate(labels1, values1, labels2, values2, languages=None):
    dupe_status = _dedupe.is_toponym_duplicate(labels1, values1, labels2, values2, languages=languages)
    return duplicate_status.from_id(dupe_status)


def is_name_duplicate_fuzzy(tokens1, scores1, tokens2, scores2, languages=None, **kw):
    dupe_status, sim = _dedupe.is_name_duplicate_fuzzy(tokens1, scores1, tokens2, scores2, languages=languages, **kw)
    return duplicate_status.from_id(dupe_status), sim


def is_street_duplicate_fuzzy(tokens1, scores1, tokens2, scores2, languages=None, **kw):
    dupe_status, sim = _dedupe.is_street_duplicate_fuzzy(tokens1, scores1, tokens2, scores2, languages=languages, **kw)
    return duplicate_status.from_id(dupe_status), sim



# -*- coding: utf-8 -*-
"""Python bindings to libpostal expand_address."""

from __future__ import unicode_literals
from postal import _expand
from postal.utils.encoding import safe_decode


def expand_address(address, languages=None, **kw):
    """
    Expand the given address into one or more normalized strings.

    Required
    --------
    @param address: the address as either Unicode or a UTF-8 encoded string

    Options
    -------
    @param languages: a tuple or list of ISO language code strings (e.g. "en", "fr", "de", etc.)
                      to use in expansion. If None is passed, use language classifier
                      to detect language automatically.
    @param address_components: an integer (bit-set) of address component expansions
                               to use e.g. ADDRESS_NAME | ADDRESS_STREET would use
                               only expansions which apply to venue names or streets.
    @param latin_ascii: use the Latin to ASCII transliterator, which normalizes e.g. æ => ae
    @param transliterate: use any available transliterators for non-Latin scripts, e.g.
                          for the Greek phrase διαφορετικούς becomes diaphoretikoús̱
    @param strip_accents: strip accented characters e.g. é => e, ç => c. This loses some
                          information in various languags, but in general we want
    @param decompose: perform Unicode normalization (NFD form)
    @param lowercase: UTF-8 lowercase the string
    @param trim_string: trim spaces on either side of the string
    @param replace_word_hyphens: add version of the string replacing hyphens with space
    @param delete_word_hyphens: add version of the string with hyphens deleted
    @param replace_numeric_hyphens: add version of the string with numeric hyphens replaced 
                                    e.g. 12345-6789 => 12345 6789
    @param delete_numeric_hyphens: add version of the string with numeric hyphens removed
                                   e.g. 12345-6789 => 123456789
    @param split_alpha_from_numeric: split tokens like CR17 into CR 17, helps with expansion
                                     of certain types of highway abbreviations
    @param delete_final_periods: remove final periods on abbreviations e.g. St. => St
    @param delete_acronym_periods: remove periods in acronyms e.g. U.S.A. => USA
    @param drop_english_possessives: normalize possessives e.g. Mark's => Marks
    @param delete_apostrophes: delete other types of hyphens e.g. O'Malley => OMalley
    @param expand_numex: converts numeric expressions e.g. Twenty sixth => 26th,
                         using either the supplied languages or the result of
                         automated language classification.
    @param roman_numerals: normalize Roman numerals e.g. IX => 9. Since these can be
                           ambiguous (especially I and V), turning this on simply
                           adds another version of the string if any potential
                           Roman numerals are found.
    """
    address = safe_decode(address, 'utf-8')
    return _expand.expand_address(address, languages=languages, **kw)


def expand_address_root(address, languages=None, **kw):
    return expand_address(address, languages=languages, root=True, **kw)


# Constants for address components
ADDRESS_NONE = _expand.ADDRESS_NONE
ADDRESS_ANY = _expand.ADDRESS_ANY
ADDRESS_NAME = _expand.ADDRESS_NAME
ADDRESS_HOUSE_NUMBER = _expand.ADDRESS_HOUSE_NUMBER
ADDRESS_STREET = _expand.ADDRESS_STREET
ADDRESS_UNIT = _expand.ADDRESS_UNIT
ADDRESS_LEVEL = _expand.ADDRESS_LEVEL
ADDRESS_STAIRCASE = _expand.ADDRESS_STAIRCASE
ADDRESS_ENTRANCE = _expand.ADDRESS_ENTRANCE
ADDRESS_CATEGORY = _expand.ADDRESS_CATEGORY
ADDRESS_NEAR = _expand.ADDRESS_NEAR
ADDRESS_TOPONYM = _expand.ADDRESS_TOPONYM
ADDRESS_POSTAL_CODE = _expand.ADDRESS_POSTAL_CODE
ADDRESS_PO_BOX = _expand.ADDRESS_PO_BOX
ADDRESS_ALL = _expand.ADDRESS_ALL



# -*- coding: utf-8 -*-
"""Test pypostal address parsing."""

from __future__ import unicode_literals

import unittest
from postal.parser import parse_address


class TestParser(unittest.TestCase):
    """Test libpostal address parser from Python."""

    def contains_components(self, address, components):
        """Test whether address parse contains specific components."""
        expected = len(components)
        got = 0

        parsed = parse_address(address)
        self.assertTrue(parsed)

        for s, c in parsed:
            if components.get(c, None) == s:
                got += 1

        self.assertEqual(expected, got)

    def test_parses(self):
        """Parser tests."""
        self.contains_components('781 Franklin Ave Crown Heights Brooklyn NYC NY 11216 USA',{
                                 'house_number': '781',
                                 'road': 'franklin ave',
                                 'suburb': 'crown heights',
                                 'city_district': 'brooklyn',
                                 'city': 'nyc',
                                 'state': 'ny',
                                 'postcode': '11216',
                                 'country': 'usa'
                                 })


if __name__ == '__main__':
    unittest.main()



from postal._token_types import *
from postal.utils.enum import Enum, EnumValue


class token_types(Enum):
    # Word types
    WORD = EnumValue(TOKEN_TYPE_WORD)
    ABBREVIATION = EnumValue(TOKEN_TYPE_ABBREVIATION)
    IDEOGRAPHIC_CHAR = EnumValue(TOKEN_TYPE_IDEOGRAPHIC_CHAR)
    HANGUL_SYLLABLE = EnumValue(TOKEN_TYPE_HANGUL_SYLLABLE)
    ACRONYM = EnumValue(TOKEN_TYPE_ACRONYM)

    # Special tokens
    EMAIL = EnumValue(TOKEN_TYPE_EMAIL)
    URL = EnumValue(TOKEN_TYPE_URL)
    US_PHONE = EnumValue(TOKEN_TYPE_US_PHONE)
    INTL_PHONE = EnumValue(TOKEN_TYPE_INTL_PHONE)

    # Numbers and numeric types
    NUMERIC = EnumValue(TOKEN_TYPE_NUMERIC)
    ORDINAL = EnumValue(TOKEN_TYPE_ORDINAL)
    ROMAN_NUMERAL = EnumValue(TOKEN_TYPE_ROMAN_NUMERAL)
    IDEOGRAPHIC_NUMBER = EnumValue(TOKEN_TYPE_IDEOGRAPHIC_NUMBER)

    # Punctuation types, may separate a phrase
    PERIOD = EnumValue(TOKEN_TYPE_PERIOD)
    EXCLAMATION = EnumValue(TOKEN_TYPE_EXCLAMATION)
    QUESTION_MARK = EnumValue(TOKEN_TYPE_QUESTION_MARK)
    COMMA = EnumValue(TOKEN_TYPE_COMMA)
    COLON = EnumValue(TOKEN_TYPE_COLON)
    SEMICOLON = EnumValue(TOKEN_TYPE_SEMICOLON)
    PLUS = EnumValue(TOKEN_TYPE_PLUS)
    AMPERSAND = EnumValue(TOKEN_TYPE_AMPERSAND)
    AT_SIGN = EnumValue(TOKEN_TYPE_AT_SIGN)
    POUND = EnumValue(TOKEN_TYPE_POUND)
    ELLIPSIS = EnumValue(TOKEN_TYPE_ELLIPSIS)
    DASH = EnumValue(TOKEN_TYPE_DASH)
    BREAKING_DASH = EnumValue(TOKEN_TYPE_BREAKING_DASH)
    HYPHEN = EnumValue(TOKEN_TYPE_HYPHEN)
    PUNCT_OPEN = EnumValue(TOKEN_TYPE_PUNCT_OPEN)
    PUNCT_CLOSE = EnumValue(TOKEN_TYPE_PUNCT_CLOSE)
    DOUBLE_QUOTE = EnumValue(TOKEN_TYPE_DOUBLE_QUOTE)
    SINGLE_QUOTE = EnumValue(TOKEN_TYPE_SINGLE_QUOTE)
    OPEN_QUOTE = EnumValue(TOKEN_TYPE_OPEN_QUOTE)
    CLOSE_QUOTE = EnumValue(TOKEN_TYPE_CLOSE_QUOTE)
    SLASH = EnumValue(TOKEN_TYPE_SLASH)
    BACKSLASH = EnumValue(TOKEN_TYPE_BACKSLASH)
    GREATER_THAN = EnumValue(TOKEN_TYPE_GREATER_THAN)
    LESS_THAN = EnumValue(TOKEN_TYPE_LESS_THAN)

    # Non-letters and whitespace
    OTHER = EnumValue(TOKEN_TYPE_OTHER)
    WHITESPACE = EnumValue(TOKEN_TYPE_WHITESPACE)
    NEWLINE = EnumValue(TOKEN_TYPE_NEWLINE)

    # Phrase, special application-level type not returned by the tokenizer
    PHRASE = 9999

    WORD_TOKEN_TYPES = set([
        WORD,
        ABBREVIATION,
        IDEOGRAPHIC_CHAR,
        HANGUL_SYLLABLE,
        ACRONYM
    ])

    NUMERIC_TOKEN_TYPES = set([
        NUMERIC,
        ORDINAL,
        ROMAN_NUMERAL,
        IDEOGRAPHIC_NUMBER,
    ])

    PUNCTUATION_TOKEN_TYPES = set([
        PERIOD,
        EXCLAMATION,
        QUESTION_MARK,
        COMMA,
        COLON,
        SEMICOLON,
        PLUS,
        AMPERSAND,
        AT_SIGN,
        POUND,
        ELLIPSIS,
        DASH,
        BREAKING_DASH,
        HYPHEN,
        PUNCT_OPEN,
        PUNCT_CLOSE,
        DOUBLE_QUOTE,
        SINGLE_QUOTE,
        OPEN_QUOTE,
        CLOSE_QUOTE,
        SLASH,
        BACKSLASH,
        GREATER_THAN,
        LESS_THAN,
    ])

    NON_ALPHANUMERIC_TOKEN_TYPES = PUNCTUATION_TOKEN_TYPES | set([
        OTHER,
        WHITESPACE,
        NEWLINE,
    ])



"""Python bindings to libpostal near_dupe_hashes."""

from postal import _near_dupe


def near_dupe_hashes(labels, values, languages=None, **kw):
    """
    Hash the given address into normalized strings that can be used to group similar
    addresses together for more detailed pairwise comparison. This can be thought of
    as the blocking function in record linkage or locally-sensitive hashing in the
    document near-duplicate detection.

    Required
    --------
    @param labels: array of component labels as either Unicode or UTF-8 encoded strings
                   e.g. ["house_number", "road", "postcode"]
    @param values: array of component values as either Unicode or UTF-8 encoded strings
                   e.g. ["123", "Broadway", "11216"]. Note len(values) must be equal to
                   len(labels).

    Options
    -------
    @param languages: a tuple or list of ISO language code strings (e.g. "en", "fr", "de", etc.)
                      to use in expansion. If None is passed, use language classifier
                      to detect language automatically.
    @param with_name: use name in the hashes
    @param with_address: use house_number & street in the hashes
    @param with_unit: use secondary unit as part of the hashes
    @param with_city_or_equivalent: use the city, city_district, suburb, or island name as one of
                                    the geo qualifiers
    @param with_small_containing_boundaries: use small containing boundaries (currently state_district)
                                             as one of the geo qualifiers
    @param with_postal_code: use postal code as one of the geo qualifiers
    @param with_latlon: use geohash + neighbors as one of the geo qualifiers
    @param latitude: latitude (Y coordinate)
    @param longitude: longitude (X coordinate)
    @param geohash_precision: geohash tile size (default = 6)
    @param name_and_address_keys: include keys with name + address + geo
    @param name_only_keys: include keys with name + geo
    @param address_only_keys: include keys with address + geo
    """
    return _near_dupe.near_dupe_hashes(labels, values, languages=languages, **kw)



# -*- coding: utf-8 -*-
"""Test pypostal expansions."""

from __future__ import unicode_literals

import unittest
from postal.expand import *


class TestExpand(unittest.TestCase):
    """Test expansions."""

    def contained_in_root_expansions(self, address, output, **kw):
        expansions = expand_address_root(address, **kw)
        self.assertTrue(expansions)

        expansions = set(expansions)
        self.assertTrue(output in expansions)

    def contained_in_expansions(self, address, output, **kw):
        """Test whether an expansion contains a particular output."""
        expansions = expand_address(address, **kw)
        self.assertTrue(expansions)

        expansions = set(expansions)
        self.assertTrue(output in expansions)

    def have_expansion_in_common(self, str1, str2, **kw):
        """Test whether strings have at least one shared expansion."""
        expansions1 = expand_address(str1, **kw)
        expansions2 = expand_address(str2, **kw)

        self.assertTrue(set(expansions1) & set(expansions2))

    def has_exact_expansions(self, address, expected_expansions, **kw):
        """Test whether an address expands exactly"""
        expansions = expand_address(address, **kw)
        self.assertTrue(expansions)

        expansions = set(expansions)
        expected_expansions = set(expected_expansions)
        self.assertTrue(expansions == expected_expansions)

    def test_expansions(self):
        """Expansion tests."""
        self.contained_in_expansions('781 Franklin Ave Crown Hts Brooklyn NY', '781 franklin avenue crown heights brooklyn new york')

        self.have_expansion_in_common('Thirty W 26th St Fl #7', '30 West Twenty-sixth Street Floor Number 7', languages=['en'])

        self.contained_in_expansions('Friedrichstraße 128, Berlin, Germany', 'friedrich strasse 128 berlin germany')

        self.contained_in_expansions('MAPLE ST.', 'maple street')
        self.contained_in_expansions('ST ISIDORE DR', 'saint isidore drive')
        self.contained_in_expansions('ST. Sebastian ST', 'saint sebastian street')
        self.contained_in_expansions("St John's St.", 'saint johns street')
        self.contained_in_expansions('MORNINGTON CR', 'mornington crescent')
        self.contained_in_expansions('Cércle rouge', 'cercle rouge')
        self.contained_in_expansions('Third St', '3rd street')

        self.contained_in_expansions('123 Dr. MLK Jr. Dr.', '123 doctor martin luther king junior drive')

        self.has_exact_expansions('120 Malcolm X Blvd', ['120 malcolm x boulevard'], roman_numerals=False)

    def test_root_expansions(self):
        self.contained_in_root_expansions("E 106TH ST", "106", address_components=ADDRESS_STREET | ADDRESS_ANY, languages=['en'])
        self.contained_in_root_expansions("PARK AVE", "park", address_components=ADDRESS_STREET | ADDRESS_ANY, languages=['en'])

if __name__ == '__main__':
    unittest.main()



import six

text_type = six.text_type
string_types = six.string_types
binary_type = six.binary_type


def safe_decode(value, encoding='utf-8', errors='strict'):
    if isinstance(value, text_type):
        return value

    if isinstance(value, (string_types, binary_type)):
        return value.decode(encoding, errors)
    else:
        return binary_type(value).decode(encoding, errors)


def safe_encode(value, incoming=None, encoding='utf-8', errors='strict'):
    if not isinstance(value, (string_types, binary_type)):
        return binary_type(value)

    if isinstance(value, text_type):
        return value.encode(encoding, errors)
    else:
        if hasattr(incoming, 'lower'):
            incoming = incoming.lower()
        if hasattr(encoding, 'lower'):
            encoding = encoding.lower()

        if value and encoding != incoming:
            value = safe_decode(value, encoding, errors)
            return value.encode(encoding, errors)
        else:
            return value
